@@ -2,8 +2,8 @@
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
 use anyhow::{anyhow, Result};
+use clap::Parser;
 use std::path::{Path, PathBuf};
-use structopt::StructOpt;
 
 use sequoia_openpgp::parse::{stream::DecryptorBuilder, Parse};
 use sequoia_openpgp::policy::StandardPolicy;
@@ -26,7 +26,7 @@ mod util;
 fn main() -> Result<(), Box<dyn std::error::Error>> {
     env_logger::init();
 
-    let cli = cli::Cli::from_args();
+    let cli = cli::Cli::parse();
 
     match cli.cmd {
         cli::Command::List {} => {
@@ -35,6 +35,9 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
         cli::Command::Status { ident, verbose } => {
             print_status(ident, verbose)?;
         }
+        cli::Command::Ssh { ident } => {
+            print_ssh(ident)?;
+        }
         cli::Command::SetIdentity { ident, id } => {
             set_identity(&ident, id)?;
         }
@@ -154,7 +157,7 @@ fn set_identity(ident: &str, id: u8) -> Result<(), Box<dyn std::error::Error>> {
 }
 
 fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
-    let mut card: Box<dyn CardBackend> = if let Some(ident) = ident {
+    let mut card: Box<dyn CardBackend + Send + Sync> = if let Some(ident) = ident {
         Box::new(util::open_card(&ident)?)
     } else {
         let mut cards = util::cards()?;
@@ -168,8 +171,6 @@ fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
     let mut pgp = OpenPgp::new(&mut *card);
     let mut open = Open::new(pgp.transaction()?)?;
 
-    let ident = open.application_identifier()?.ident();
-
     print!("OpenPGP card {}", open.application_identifier()?.ident());
 
     let ai = open.application_identifier()?;
@@ -206,7 +207,12 @@ fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
     }
 
     if let Some(lang) = crd.lang() {
-        println!("Language preferences '{:?}'", lang);
+        let l = lang
+            .iter()
+            .map(|l| format!("{}", l))
+            .collect::<Vec<_>>()
+            .join(", ");
+        println!("Language preferences: '{}'", l);
     }
 
     // information about subkeys
@@ -223,7 +229,6 @@ fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
         println! {"  created: {}", kgt.formatted()};
     }
     println! {"  algorithm: {}", open.algorithm_attributes(KeyType::Signing)?};
-
     if verbose {
         if let Ok(pkm) = open.public_key(KeyType::Signing) {
             println! {"  public key material: {}", pkm};
@@ -239,7 +244,6 @@ fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
         println! {"  created: {}", kgt.formatted()};
     }
     println! {"  algorithm: {}", open.algorithm_attributes(KeyType::Decryption)?};
-
     if verbose {
         if let Ok(pkm) = open.public_key(KeyType::Decryption) {
             println! {"  public key material: {}", pkm};
@@ -251,20 +255,12 @@ fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
     if let Some(fp) = fps.authentication() {
         println!("  fingerprint: {}", fp.to_spaced_hex());
     }
-    let pubkey = open.public_key(KeyType::Authentication);
-    if let Ok(pkm) = &pubkey {
-        if let Ok(ssh) = util::get_ssh_pubkey_string(pkm, ident) {
-            // print auth key as openssh public key string
-            println!("  {}", ssh);
-        }
-    }
-
     if let Some(kgt) = kgt.authentication() {
         println! {"  created: {}", kgt.formatted()};
     }
     println! {"  algorithm: {}", open.algorithm_attributes(KeyType::Authentication)?};
     if verbose {
-        if let Ok(pkm) = pubkey {
+        if let Ok(pkm) = open.public_key(KeyType::Authentication) {
             println! {"  public key material: {}", pkm};
         }
     }
@@ -312,6 +308,44 @@ fn print_status(ident: Option<String>, verbose: bool) -> Result<()> {
     Ok(())
 }
 
+fn print_ssh(ident: Option<String>) -> Result<()> {
+    let mut card: Box<dyn CardBackend + Send + Sync> = if let Some(ident) = ident {
+        Box::new(util::open_card(&ident)?)
+    } else {
+        let mut cards = util::cards()?;
+        if cards.len() == 1 {
+            Box::new(cards.pop().unwrap())
+        } else {
+            return Err(anyhow::anyhow!("Found {} cards", cards.len()));
+        }
+    };
+
+    let mut pgp = OpenPgp::new(&mut *card);
+    let mut open = Open::new(pgp.transaction()?)?;
+
+    let ident = open.application_identifier()?.ident();
+
+    println!("OpenPGP card {}", open.application_identifier()?.ident());
+
+    // Print fingerprint of authentication subkey
+    let fps = open.fingerprints()?;
+
+    println!();
+    if let Some(fp) = fps.authentication() {
+        println!("Authentication key fingerprint:\n{}", fp);
+    }
+
+    // Show authentication subkey as openssh public key string
+    if let Ok(pkm) = open.public_key(KeyType::Authentication) {
+        if let Ok(ssh) = util::get_ssh_pubkey_string(&pkm, ident) {
+            println!();
+            println!("Authentication key as ssh public key:\n{}", ssh);
+        }
+    }
+
+    Ok(())
+}
+
 fn decrypt(
     ident: &str,
     pin_file: Option<PathBuf>,
@@ -321,7 +355,7 @@ fn decrypt(
     let p = StandardPolicy::new();
     let cert = Cert::from_file(cert_file)?;
 
-    let input = util::open_or_stdin(input.as_deref())?;
+    let input = util::open_or_stdin(input)?;
 
     let mut card = util::open_card(ident)?;
     let mut pgp = OpenPgp::new(&mut card);
@@ -347,7 +381,7 @@ fn sign_detached(
 ) -> Result<(), Box<dyn std::error::Error>> {
     let cert = Cert::from_file(cert_file)?;
 
-    let mut input = util::open_or_stdin(input.as_deref())?;
+    let mut input = util::open_or_stdin(input)?;
 
     let mut card = util::open_card(ident)?;
     let mut pgp = OpenPgp::new(&mut card);
